#走行中画面
import wx
from datetime import datetime,timedelta,timezone
import time
import subprocess
import RPi.GPIO as GPIO
import threading
from socket import socket,AF_INET,SOCK_STREAM

from .get_status import GetDate
from .animation import RunAnimation,CountUp
from .gpio_setup import OK,RIGHT
from .finish_frame import FinishFrame
from .emergency import Emergency

EVT_COMM_ID=wx.NewIdRef()

HOST='localhost'
PORT_R=51001    #C->send
MAX_R=1024
connection_r=(HOST,PORT_R)

ckfinish=False
ckem=False

class CommEvent(wx.PyEvent):    #通信スレッドのイベント処理
    def __init__(self,flag):
        wx.PyEvent.__init__(self)
        self.SetEventType(EVT_COMM_ID)
        self.flag=flag

class CommunicationThread(threading.Thread):    #通信スレッド
    def __init__(self,window):
        threading.Thread.__init__(self)
        self._window=window
        self.start()

    def run(self):
        global ckfinish,ckem
        socket_r=socket(AF_INET,SOCK_STREAM)
        socket_r.bind(connection_r)
        socket_r.listen()
        print('recv ready')
        conn,addr=socket_r.accept()
        print('accept')     #ここでCからの受信待ち
        while True:
            try:
                data1=conn.recv(MAX_R)
                print(data1)
                mode=data1.decode()
                print(mode)
                if(mode=='f'):  #正常終了
                    ckfinish=True
                    wx.PostEvent(self._window,CommEvent(mode))
                    break
                elif(mode=='e'):    #緊急停止
                    ckem=True
                    wx.PostEvent(self._window,CommEvent(mode))
                    break
                else:
                    pass
            except:
                #通信失敗した場合の処理
                print('error')
                mode='g'
                wx.PostEvent(self._window,CommEvent(mode))
                break
        conn.close()
        socket_r.close()

class RunFrame(wx.Frame):
    def __init__(self,parent):
        wx.Frame.__init__(self,parent,-1,"run")
        size=wx.ScreenDC().GetSize()
        self.SetSize(size)
        self.SetPosition(wx.Point(0,0))
        self.SetRunFrame()

    def ShowFinish(self):
        self.timer.Stop()
        finishFrame=FinishFrame(self)
        finishID=finishFrame.Show()

    def ShowEmergency(self):
        self.timer.Stop()
        emergency=Emergency(self)
        emrgencyID=emergency.Show()

    def OnTime(self,event):
        if not self.worker:
            self.worker=CommunicationThread(self)
        now=GetDate()
        status=now
        self.status.SetLabel("%s"%status)
        CountUp()
        newfig=RunAnimation()
        self.figure.SetBitmap(newfig)
        self.Refresh()

    def OnComm(self,event):
        if(event.flag=='f'):
            self.ShowFinish()
        elif(event.flag=='e'):
            self.ShowEmergency()
        elif(event.flag=='g'):
            wx.Exit()
            subprocess.run(['sudo','reboot'])

    def SetRunFrame(self):
        runpanel=wx.Panel(self)
        blank1=wx.StaticText(runpanel,-1,"")
        statustext=GetDate()
        self.status=wx.StaticText(runpanel,-1,statustext)
        message=wx.StaticText(runpanel,-1,"走行中......")
        fig=RunAnimation()
        self.figure=wx.StaticBitmap(runpanel,-1,fig)
        self.worker=None    #スレッドを立てたかどうかのチェック用

        font1=wx.Font(30,wx.FONTFAMILY_DEFAULT,wx.FONTSTYLE_NORMAL,wx.FONTWEIGHT_NORMAL)
        self.status.SetFont(font1)
        font2=wx.Font(60,wx.FONTFAMILY_DEFAULT,wx.FONTSTYLE_NORMAL,wx.FONTWEIGHT_NORMAL)
        message.SetFont(font2)

        self.timer=wx.Timer(self)
        self.Bind(wx.EVT_TIMER,self.OnTime)
        self.timer.Start(1000,wx.TIMER_CONTINUOUS)
        self.Connect(-1,-1,EVT_COMM_ID,self.OnComm)

        sizer=wx.GridBagSizer()
        sizer.Add(blank1,(0,0),(1,1),flag=wx.EXPAND)
        sizer.Add(self.status,(0,1),(1,1),flag=wx.EXPAND|wx.LEFT,border=700)
        sizer.Add(message,(1,0),(1,2),flag=wx.EXPAND|wx.LEFT,border=300)
        sizer.Add(self.figure,(2,0),(1,2),flag=wx.EXPAND)
        sizer.AddGrowableRow(0,1)
        sizer.AddGrowableRow(1,2)
        sizer.AddGrowableRow(2,2)
        sizer.AddGrowableCol(0,1)
        sizer.AddGrowableCol(1,1)
        runpanel.SetSizer(sizer)
